﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Data;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Runtime.Remoting;
using System.Text;
using System.Windows.Forms;
using MetaFiler.Data;
using MetaFiler.Utils;

namespace MetaFiler.Controls
{
    public partial class ExplorerUserControl : UserControlBase
    {
        #region Declarations

        private delegate void SelectTagsValueCellHandler();

        #endregion

        #region Variables

        private bool ignoreDataNodeChange = false;
        private bool ignoreNodeCheck = false;

        private SelectTagsValueCellHandler selectTagsValueCellHandler;

        #endregion

        #region Constructor / Destructor

        public ExplorerUserControl()
        {
            InitializeComponent();
            tagsBindingSource.DataSource = Program.metaDataSet;
            dirsBindingSource.DataSource = Program.metaDataSet;
            filesBindingSource.DataSource = Program.metaDataSet;
            selectTagsValueCellHandler = new SelectTagsValueCellHandler(SelectTagsValueCell);
            tagsDataGridView.Enabled = false;
        }

        protected override void Dispose(bool disposing)
        {
            selectTagsValueCellHandler = null;
            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        #endregion

        #region Methods

        private void SelectTagsValueCell()
        {
            if (tagsDataGridView.CurrentRow != null)
            {
                DataGridViewCell valCell = tagsDataGridView.CurrentRow.Cells[valueDataGridViewTextBoxColumn.Name];
                if (!valCell.Selected)
                    valCell.Selected = true;
            }
        }

        private TreeNode FindDirectoryFileNodeByPath(TreeNode dirNode, string path)
        {
            foreach (TreeNode tNode in dirNode.Nodes)
            {
                if (tNode.Tag is MetaDataSet.FilesRow)
                {
                    MetaDataSet.FilesRow dfRow = (MetaDataSet.FilesRow) tNode.Tag;
                    if (dfRow.Path == path)
                        return tNode;
                }
            }

            return null;
        }

        private TreeNode FindTreeNodeByDirectoryID(int dirID)
        {
            foreach (TreeNode tn in explorerTreeView.Nodes)
            {
                if (tn.Tag is DataRowView)
                {
                    MetaDataSet.DirectoriesRow dirRow = (MetaDataSet.DirectoriesRow) ((DataRowView) tn.Tag).Row;
                    if (dirRow.ID == dirID)
                        return tn;
                }
            }
            return null;
        }

        private void _GetCheckedNodes(TreeNode parentNode, ref List<TreeNode> chNodes)
        {
            foreach (TreeNode chNode in parentNode.Nodes)
            {
                if (chNode.Checked)
                    chNodes.Add(chNode);

                _GetCheckedNodes(chNode, ref chNodes);
            }
        }

        private List<TreeNode> GetCheckedNodes()
        {
            List<TreeNode> chNodes = new List<TreeNode>();
            foreach (TreeNode tNode in explorerTreeView.Nodes)
            {
                if (tNode.Checked)
                {
                    chNodes.Add(tNode);
                }
                _GetCheckedNodes(tNode, ref chNodes);
            }
            return chNodes;
        }

        private void DeleteNode(TreeNode tNode, bool confirmDelete = true, string delName = "item", bool doSave = true, bool saveDirs = true)
        {
            if (tNode != null)
            {
                if (tNode.Tag is MetaDataSet.FilesRow)
                {
                    if (!tNode.Parent.Checked)
                    {
                        if (Misc.ConfirmItemDelete(delName, !confirmDelete))
                        {
                            MetaDataSet.FilesRow fRow = (MetaDataSet.FilesRow)tNode.Tag;
                            int fdirID = fRow.DirectoryID;
                            MetaDataSet.DirectoriesRow cfDirRow = ((MetaDataSet.DirectoriesRow) fRow.GetParentRow("FK_Directories_Files"));
                            cfDirRow.FileCount = cfDirRow.FileCount - 1;
                            fRow.Delete();

                            if (cfDirRow.FileCount == 0)
                            {
                                TreeNode cfdirNode = FindTreeNodeByDirectoryID(cfDirRow.ID);
                                if (cfdirNode != null)
                                {
                                    cfdirNode.ImageIndex = 0;
                                    cfdirNode.SelectedImageIndex = 0;
                                }
                            }
                            
                            if (doSave)
                            {
                                Program.DataSave("files_" + fdirID);
                                if (saveDirs)
                                    Program.DataSave("dirs");
                            }
                            tNode.Remove();
                        }                        
                    }
                }
                else if (tNode.Tag is DataRowView)
                {
                    DataRowView sRow = (DataRowView)tNode.Tag;
                    if (Misc.ConfirmItemDelete(delName, !confirmDelete))
                    {
                        if (sRow.Row is MetaDataSet.DirectoriesRow)
                        {
                            MetaDataSet.DirectoriesRow dRow = (MetaDataSet.DirectoriesRow)sRow.Row;

                            if (tNode.Nodes.Count == 1)
                            {
                                if (tNode.Nodes[0].Tag is string)
                                {
                                    if (tNode.Nodes[0].Tag.ToString() == "DirFilesDummy")
                                    {
                                        tNode.Nodes.Clear();
                                        doSave = true;
                                    }
                                }
                            }

                            int dirID = dRow.ID;
                            dRow.Delete();
                            if (doSave)
                            {
                                Program.DataSave("files_" + dirID);
                                if (saveDirs)
                                    Program.DataSave("dirs");
                            }
                            tNode.Remove();
                        }
                    }
                }                
            }
        }

        private void DeleteSelectedItem()
        {
            if (!multiSelectToolStripButton.Checked)
            {
                DeleteNode(explorerTreeView.SelectedNode);
            }
            else
            {
                List<TreeNode> chNodes = GetCheckedNodes();
                if (chNodes.Count > 0)
                {
                    if (Misc.ConfirmItemDelete("selected items"))
                    {
                        while (chNodes.Count > 0)
                        {
                            TreeNode chNode = chNodes[0];
                            if (chNodes.Count == 1)
                            {
                                DeleteNode(chNode, false, "", true, false);
                            }
                            else
                            {
                                DeleteNode(chNode, false, "", true, false);
                            }
                            chNodes.RemoveAt(0);
                        }
                    }
                }
                
                Program.DataSave("dirs");
            }

            explorerDeleteToolStripButton.Enabled = (explorerTreeView.SelectedNode != null) || multiSelectToolStripButton.Checked;
            tagsDataGridView.Enabled = explorerDeleteToolStripButton.Enabled && dirsBindingSource.Count > 0;
        }

        private DataRowView GetDirRowByIndex(int rowIndex)
        {
            return (DataRowView) dirsBindingSource[rowIndex];
        }

        #endregion

        #region Events

        private void dirsBindingSource_ListChanged(object sender, ListChangedEventArgs e)
        {

            Debug.WriteLine("Dir: " + e.ListChangedType);

            DataRowView dirRow = null;
            switch (e.ListChangedType)
            {
                case ListChangedType.ItemAdded:
                    dirRow = GetDirRowByIndex(e.NewIndex);
                    TreeNode dirNode = new TreeNode(dirRow["Name"].ToString(), 0, 0);
                    if (Convert.ToInt32(dirRow.Row["FileCount"]) > 0)
                    {
                        dirNode.ImageIndex = 1;
                        dirNode.SelectedImageIndex = 1;

                        TreeNode dfDummyNode = dirNode.Nodes.Add("DirFilesDummy");
                        dfDummyNode.Tag = "DirFilesDummy";
                    }
                    dirNode.Tag = dirRow;
                    explorerTreeView.Nodes.Add(dirNode);
                    break;
                case ListChangedType.Reset:
                    explorerTreeView.Nodes.Clear();
                    foreach (DataRowView cdRow in dirsBindingSource)
                    {
                        TreeNode cdNode = new TreeNode(cdRow["Name"].ToString(), 0, 0);
                        if (Convert.ToInt32(cdRow.Row["FileCount"]) > 0)
                        {
                            cdNode.ImageIndex = 1;
                            cdNode.SelectedImageIndex = 1;

                            TreeNode dfDummyNode = cdNode.Nodes.Add("DirFilesDummy");
                            dfDummyNode.Tag = "DirFilesDummy";
                        }
                        cdNode.Tag = cdRow;
                        explorerTreeView.Nodes.Add(cdNode);
                    }
                    break;                    
                case ListChangedType.ItemChanged:
                    dirRow = GetDirRowByIndex(e.NewIndex);
                    MetaDataSet.DirectoriesRow chdirRow = (MetaDataSet.DirectoriesRow) dirRow.Row;
                    TreeNode cdirNode = FindTreeNodeByDirectoryID(chdirRow.ID);
                    MetaDataSet.FilesRow[] cdFileRows = chdirRow.GetFilesRows();
                    foreach (MetaDataSet.FilesRow cdFileRow in cdFileRows)
                    {
                        TreeNode cfNode = FindDirectoryFileNodeByPath(cdirNode, cdFileRow.Path);
                        if (cfNode == null)
                        {
                            cfNode = new TreeNode(cdFileRow.Name, 2, 2);
                            cfNode.Tag = cdFileRow;
                            cdirNode.Nodes.Add(cfNode);
                        }
                    }
                    if (cdFileRows.Length > 0)
                    {
                        cdirNode.ImageIndex = 1;
                        cdirNode.SelectedImageIndex = 1;
                    }
                    break;
                default:
                    
                    break;                    
            }
        }
        private void filesBindingSource_ListChanged(object sender, ListChangedEventArgs e)
        {
            Debug.WriteLine("File: " + e.ListChangedType);


            switch (e.ListChangedType)
            {
                case ListChangedType.ItemAdded:
                    if (!ignoreDataNodeChange)
                    {
                        MetaDataSet.FilesRow fRow = (MetaDataSet.FilesRow)((DataRowView)filesBindingSource[e.NewIndex]).Row;

                        TreeNode dirNode = FindTreeNodeByDirectoryID(fRow.DirectoryID);
                        if (dirNode != null)
                        {
                            TreeNode fileNode = new TreeNode(fRow.Name, 2, 2);
                            fileNode.Tag = fRow;
                            dirNode.Nodes.Add(fileNode);
                            dirNode.ExpandAll();
                            explorerTreeView.SelectedNode = fileNode;
                            if (Null.IntFromNull(((DataRowView)dirNode.Tag)["FileCount"]) > 0)
                            {
                                dirNode.ImageIndex = 1;
                                dirNode.SelectedImageIndex = 1;
                            }

                        }    
                    }
                    

                    break;                
                default:
                    break;
            }
        }

        private void explorerTreeView_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            if (ignoreDataNodeChange)
                return;

            string nodeTagStr = "";
            if (e.Node.Tag is DataRowView)
            {
                if (e.Node.Nodes.Count > 0)
                {
                    if (e.Node.Nodes[0].Tag is string)
                        nodeTagStr = e.Node.Nodes[0].Tag.ToString();
                }

                if (nodeTagStr.Length > 0)
                {
                    DataRowView nodeRow = (DataRowView)e.Node.Tag;
                    if (nodeRow.Row is MetaDataSet.DirectoriesRow)
                    {
                        ignoreDataNodeChange = true;

                        MetaDataSet.DirectoriesRow dirRow = (MetaDataSet.DirectoriesRow)nodeRow.Row;
                        if (nodeTagStr == "DirFilesDummy")
                        {
                            e.Node.Nodes.Clear();
                            Program.DataLoad("files_" + dirRow.ID);
                            Program.DataLoad("filestags_" + dirRow.ID);

                            filesBindingSource.Filter = "DirectoryID = " + dirRow.ID;

                            foreach (MetaDataSet.FilesRow dirFileRow in dirRow.GetFilesRows())
                            {
                                TreeNode fileNode = new TreeNode(dirFileRow.Name, 2, 2);
                                fileNode.Tag = dirFileRow;
                                e.Node.Nodes.Add(fileNode);
                            }
                            e.Node.ExpandAll();
                        }

                        ignoreDataNodeChange = false;
                    }                    
                }
            }
            else if (e.Node.Tag is string)
            {
                nodeTagStr = e.Node.Tag.ToString();
            }
        }

        private void explorerTreeView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                DeleteSelectedItem();
            }
        }

        private void multiSelectToolStripButton_CheckedChanged(object sender, EventArgs e)
        {
            explorerTreeView.CheckBoxes = multiSelectToolStripButton.Checked;
            explorerDeleteToolStripButton.Enabled = (explorerTreeView.SelectedNode != null) || multiSelectToolStripButton.Checked;
            tagsDataGridView.Enabled = explorerDeleteToolStripButton.Enabled && dirsBindingSource.Count > 0;            
        }

        private void explorerTreeView_AfterCheck(object sender, TreeViewEventArgs e)
        {
            if (ignoreNodeCheck)
                return;

            if (e.Node.Tag is DataRowView)
            {
                DataRowView nDataRowView = (DataRowView)e.Node.Tag;
                if (nDataRowView.Row is MetaDataSet.DirectoriesRow)
                {
                    MetaDataSet.DirectoriesRow dRow = (MetaDataSet.DirectoriesRow)nDataRowView.Row;
                    TreeNode dirTreeNode = FindTreeNodeByDirectoryID(dRow.ID);
                    foreach (TreeNode dfNode in dirTreeNode.Nodes)
                    {
                        dfNode.Checked = e.Node.Checked;
                    }
                }
            }
            else if (e.Node.Tag is MetaDataSet.FilesRow)
            {
                MetaDataSet.FilesRow fRow = (MetaDataSet.FilesRow)e.Node.Tag;
                if (!e.Node.Checked)
                {
                    ignoreNodeCheck = true;
                    e.Node.Parent.Checked = false;
                    ignoreNodeCheck = false;
                }
            }
        }
        private void explorerTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            explorerDeleteToolStripButton.Enabled = (explorerTreeView.SelectedNode != null) || multiSelectToolStripButton.Checked;
            tagsDataGridView.Enabled = explorerDeleteToolStripButton.Enabled;

            if (explorerDeleteToolStripButton.Enabled)
            {
                TreeNode selNode = explorerTreeView.SelectedNode;
                bool isDir = false;
                MetaDataSet.DirectoriesRow snDirRow = null;
                MetaDataSet.FilesRow snFileRow = null;
                if (selNode.Tag is DataRowView)
                {
                    DataRowView snRowView = (DataRowView) selNode.Tag;
                    if (snRowView.Row is MetaDataSet.DirectoriesRow)
                    {
                        snDirRow = (MetaDataSet.DirectoriesRow) snRowView.Row;
                        tagsBindingSource.Filter = "Usage = 'Directories'";
                        isDir = true;
                    }
                }
                else if (selNode.Tag is MetaDataSet.FilesRow)
                {
                    snFileRow = (MetaDataSet.FilesRow) selNode.Tag;
                    snDirRow = (MetaDataSet.DirectoriesRow)snFileRow.GetParentRow("FK_Directories_Files");
                    tagsBindingSource.Filter = "Usage = 'Files'";
                    isDir = false;
                }
                tagsBindingSource_ListChanged(tagsBindingSource, new ListChangedEventArgs(ListChangedType.Reset, 0));

                foreach (TagsDataSet.TagsRow ctRow in tagsDataSet.Tags)
                {
                    if (isDir)
                    {
                        MetaDataSet.DirectoryTagsRow cdtRow = Program.FindDirectoryTagsRow(snDirRow.ID, ctRow.TagID);
                        if (cdtRow != null)
                        {
                            ctRow["Value"] = cdtRow["Value"];
                        }
                        else
                        {
                            ctRow.Value = "";
                        }
                    }
                    else
                    {
                        MetaDataSet.FileTagsRow cftRow = Program.FindFileTagsRow(snFileRow.ID, ctRow.TagID);
                        if (cftRow != null)
                        {
                            ctRow["Value"] = cftRow["Value"];
                        }
                        else
                        {
                            ctRow.Value = "";
                        }                        
                    }
                }
            }
        }

        private void explorerDeleteToolStripButton_Click(object sender, EventArgs e)
        {
            DeleteSelectedItem();
        }

        private void tagsBindingSource_ListChanged(object sender, ListChangedEventArgs e)
        {
            tagsDataSet.Tags.Clear();
            foreach (DataRowView tRowView in tagsBindingSource)
            {
                DataRow[] ctfRows = tagsDataSet.Tags.Select("TagID = " + tRowView["ID"]);
                TagsDataSet.TagsRow tRow;
                if (ctfRows.Length > 0)
                {
                    tRow = (TagsDataSet.TagsRow) ctfRows[0];
                }
                else
                {
                    tRow = tagsDataSet.Tags.NewTagsRow();
                    tagsDataSet.Tags.AddTagsRow(tRow);
                }
                tRow.Name = tRowView["Name"].ToString();
                tRow.DataType = tRowView["DataType"].ToString();
                tRow.TagID = Convert.ToInt32(tRowView["ID"]);                
                tRow["DataMin"] = tRowView["DataMin"];
                tRow["DataMax"] = tRowView["DataMax"];
                tRow.Value = "";

            }
        }

        private void tagsDataGridView_RowEnter(object sender, DataGridViewCellEventArgs e)
        {
            if (tagsDataGridView.Columns[e.ColumnIndex].Name == valueDataGridViewTextBoxColumn.Name)
            {

            }
            else
            {
                tagsDataGridView.BeginInvoke(selectTagsValueCellHandler);
            }
        }

        private void tagsDataGridView_CellEnter(object sender, DataGridViewCellEventArgs e)
        {
            if (tagsDataGridView.Columns[e.ColumnIndex].Name == valueDataGridViewTextBoxColumn.Name)
            {

            }
            else
            {
                tagsDataGridView.BeginInvoke(selectTagsValueCellHandler);
            }
        }

        private void tagsDataGridView_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            tagsDataGridView.BeginEdit(true);
        }

        private void tagsDataGridView_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            DataRowView curRowView = (DataRowView)tagsDataGridView.Rows[e.RowIndex].DataBoundItem;
            string colName = tagsDataGridView.Columns[e.ColumnIndex].DataPropertyName;
            string dataType = curRowView["DataType"].ToString();
            int dataMin = Null.IntFromNull(curRowView["DataMin"], int.MinValue);
            int dataMax = Null.IntFromNull(curRowView["DataMax"], int.MaxValue);
            if (dataType == "Integer")
            {
                double cd = 0;
                if (double.TryParse(curRowView[colName].ToString(), NumberStyles.Integer, CultureInfo.CurrentCulture,
                    out cd))
                {
                    int civ = Misc.LimitValue((int)cd, dataMin, dataMax);
                    curRowView["Value"] = civ.ToString();
                }
                else
                {
                    curRowView["Value"] = "";
                }
            }
            else if (dataType == "Float")
            {
                double cd = 0;
                if (double.TryParse(curRowView[colName].ToString(), NumberStyles.Float, CultureInfo.CurrentCulture, out cd))
                {
                    float cfv = Misc.LimitValue((float)cd, dataMin, dataMax);
                    curRowView["Value"] = cfv.ToString("n2");
                }
                else
                {
                    curRowView["Value"] = cd.ToString("n2");
                }
            }

            MetaDataSet.DirectoriesRow selDirRow = null;
            MetaDataSet.FilesRow selFileRow = null;
            bool isDir = false;
            if (explorerTreeView.SelectedNode.Tag is DataRowView)
            {
                DataRowView snDataRowView = (DataRowView) explorerTreeView.SelectedNode.Tag;
                if (snDataRowView.Row is MetaDataSet.DirectoriesRow)
                {
                    selDirRow = (MetaDataSet.DirectoriesRow)snDataRowView.Row;
                    isDir = true;
                }
            }
            else if (explorerTreeView.SelectedNode.Tag is MetaDataSet.FilesRow)
            {
                selFileRow = (MetaDataSet.FilesRow) explorerTreeView.SelectedNode.Tag;
                selDirRow = (MetaDataSet.DirectoriesRow)selFileRow.GetParentRow("FK_Directories_Files");
            }

            foreach (TagsDataSet.TagsRow tagRow in tagsDataSet.Tags)
            {
                int tagID = Null.IntFromNull(tagRow["TagID"], -1);
                string valStr = Null.StringFromNull(tagRow["Value"]).Trim();                
                MetaDataSet.DirectoryTagsRow dtRow;
                MetaDataSet.FileTagsRow ftRow;

                if (isDir)
                {
                    dtRow = Program.FindDirectoryTagsRow(selDirRow.ID, tagID);
                    if (dtRow != null)
                    {
                        if (valStr.Length == 0)
                        {
                            dtRow.Delete();
                        }
                        else
                        {
                            dtRow.Value = valStr;
                        }
                    }
                    else
                    {
                        if (valStr.Length > 0)
                        {
                            dtRow = Program.metaDataSet.DirectoryTags.NewDirectoryTagsRow();
                            dtRow.DirectoryID = selDirRow.ID;
                            dtRow.TagID = tagID;
                            dtRow.Value = valStr;
                            Program.metaDataSet.DirectoryTags.AddDirectoryTagsRow(dtRow);
                        }
                    }
                    Program.DataSave("dirs");
                }
                else
                {
                    ftRow = Program.FindFileTagsRow(selFileRow.ID, tagID);
                    if (ftRow != null)
                    {
                        if (valStr.Length == 0)
                        {
                            ftRow.Delete();
                        }
                        else
                        {
                            ftRow.Value = valStr;
                        }
                    }
                    else
                    {
                        if (valStr.Length > 0)
                        {
                            ftRow = Program.metaDataSet.FileTags.NewFileTagsRow();
                            ftRow.DirectoryID = selDirRow.ID;
                            ftRow.FileID = selFileRow.ID;
                            ftRow.TagID = tagID;
                            ftRow.Value = valStr;
                            Program.metaDataSet.FileTags.AddFileTagsRow(ftRow);
                        }
                    }
                    Program.DataSave("dirs");
                    Program.DataSave("filestags_" + selDirRow.ID);
                }

            }
        }

        private void explorerTreeView_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left && e.Clicks == 1)
            {
                TreeViewHitTestInfo htInfo = explorerTreeView.HitTest(e.Location);
                if (htInfo.Location == TreeViewHitTestLocations.Image)
                {
                    if (htInfo.Node.IsSelected)
                    {
                        if (!htInfo.Node.IsExpanded)
                        {
                            htInfo.Node.Expand();
                        }
                        else
                        {
                            htInfo.Node.Collapse();
                        }                        
                    }
                }
            }
        }

        private void tagsDataGridView_EnabledChanged(object sender, EventArgs e)
        {
            if (!tagsDataGridView.Enabled)
            {
                tagsDataGridView.DataMember = "";
                tagsDataGridView.BackColor = Color.DimGray;
                tagsDataGridView.RowsDefaultCellStyle.BackColor = Color.Silver;
                tagsDataGridView.RowsDefaultCellStyle.SelectionBackColor = Color.Silver;
                tagsDataGridView.CellBorderStyle = DataGridViewCellBorderStyle.None;
            }
            else
            {
                tagsDataGridView.DataMember = "Tags";
                tagsDataGridView.BackColor = Color.Silver;
                tagsDataGridView.RowsDefaultCellStyle.BackColor = Color.White;
                tagsDataGridView.RowsDefaultCellStyle.SelectionBackColor = Color.Maroon;
                tagsDataGridView.CellBorderStyle = DataGridViewCellBorderStyle.Single;
            }
        }

        #endregion
    }
}
