﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MetaFiler.Utils;

namespace MetaFiler.Controls
{
    public partial class TagsUserControl : UserControlBase
    {
        #region Constructor

        public TagsUserControl()
        {
            InitializeComponent();
            dataTypesBindingSource.DataSource = Program.metaDataSet;
            tagsBindingSource.DataSource = Program.metaDataSet;
        }

        #endregion

        #region Overrides

        public override void TabPageClosing(CancelEventArgs e)
        {
            base.TabPageClosing(e);
            Program.DataSave("tags");
        }

        #endregion

        #region Events
        private void tagsDataGridView_NewRowNeeded(object sender, DataGridViewRowEventArgs e)
        {
            DataRowView dtRow = ((DataRowView) dataTypesBindingSource.List[0]);
            ((DataRowView) ((DataGridViewRow) e.Row).DataBoundItem)["Usage"] = "Directories";
            ((DataRowView) ((DataGridViewRow) e.Row).DataBoundItem)["DataType"] = dtRow["Name"];
        }

        private void tagsDataGridView_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            if (tagsDataGridView.Columns[e.ColumnIndex] == nameDataGridViewTextBoxColumn || tagsDataGridView.Columns[e.ColumnIndex] == usageDataGridViewTextBoxColumn)
            {
                if (e.Exception.Message.Contains("already present"))
                {
                    MessageBox.Show("Tag with same usage and name already exists!", "Tag Edit", MessageBoxButtons.OK, MessageBoxIcon.Error);                    
                }
            }
            else if (tagsDataGridView.Columns[e.ColumnIndex] == dataMinDataGridViewTextBoxColumn ||
                     tagsDataGridView.Columns[e.ColumnIndex] == dataMaxDataGridViewTextBoxColumn)
            {
                if (e.Exception.Message.Contains("correct format"))
                {
                    MessageBox.Show("Min & max data values can only contain whole numbers!", "Tag Edit", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            e.ThrowException = false;
        }

        private void tagsDataGridView_UserDeletingRow(object sender, DataGridViewRowCancelEventArgs e)
        {
            if (
                MessageBox.Show("Are you sure you want to delete selected tag?", "Tag Edit", MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.No)
            {
                e.Cancel = true;
            }
        }

        #endregion

        private void tagsDataGridView_CellPainting(object sender, DataGridViewCellPaintingEventArgs e)
        {
            if (e.RowIndex == tagsDataGridView.NewRowIndex)
            {
                e.Graphics.FillRectangle(Brushes.LightSkyBlue, e.CellBounds);
                Rectangle cbRect = e.CellBounds;
                //e.Graphics.DrawRectangle(Pens.DarkSlateGray, cbRect);

                if (e.ColumnIndex == usageDataGridViewTextBoxColumn.DisplayIndex)
                {
                    if (tagsDataGridView.CurrentRow != tagsDataGridView.Rows[tagsDataGridView.NewRowIndex])
                    {
                        e.Graphics.DrawString("Add New Tag...", Font, Brushes.Black, e.CellBounds, new StringFormat(StringFormatFlags.NoWrap) { Alignment = StringAlignment.Near, LineAlignment = StringAlignment.Center });                        
                    }
                }
                else
                {
                    //e.PaintContent(e.CellBounds);
                }

                e.Handled = true;
            }
            else
            {
                e.Paint(e.CellBounds, e.PaintParts);
            }
        }

        private void tagsDataGridView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                if (tagsDataGridView.CurrentRow != null)
                {
                    if (Misc.ConfirmItemDelete("selected tag"))
                    {
                        DataRowView tagRowView = (DataRowView) tagsDataGridView.CurrentRow.DataBoundItem;
                        tagRowView.Delete();
                        Program.DataSave("tags");
                        Program.DataSave("dirs");
                        Program.DataSave("files");
                    }
                }
            }
        }

    }
}
