﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using MetaFiler.Controls;
using MetaFiler.Data;
using MetaFiler.Utils;

namespace MetaFiler.Forms
{
    public partial class MainForm : Form
    {
        #region Variables

        internal int tabPageImageKey = 0;
        internal bool confirmExit = true;

        #endregion

        #region Constructor

        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        #region Methods
        public void Add(string addType, string addPath, bool childDirectory = false)
        {
            if (addType == "file")
            {
                string dir = Misc.GetDirectoryFromPath(addPath);
                MetaDataSet.DirectoriesRow dirRow = Program.FindDirByPath(dir);
                if (dirRow == null)
                {
                    dirRow = Program.metaDataSet.Directories.NewDirectoriesRow();
                    dirRow.Path = dir;
                    dirRow.Name = Misc.GetDirectoryNameFromPath(dir);
                    Program.metaDataSet.Directories.AddDirectoriesRow(dirRow);
                }

                MetaDataSet.FilesRow fileRow = Program.FindFileByPath(dirRow.ID, addPath);
                if (fileRow == null)
                {
                    dirRow.FileCount = dirRow.FileCount + 1;
                    fileRow = Program.metaDataSet.Files.NewFilesRow();
                    fileRow.DirectoryID = dirRow.ID;
                    fileRow.Path = addPath;
                    fileRow.Name = Misc.GetFileNameFromPath(addPath, false);
                    Program.metaDataSet.Files.AddFilesRow(fileRow);
                }

                Program.DataSave("dirs");
                Program.DataSave("files_" + dirRow.ID);
            }
            else if (addType == "dir")
            {                
                string dir = Misc.GetDirectoryFromPath(addPath.TrimEnd('\\') + '\\');
                MetaDataSet.DirectoriesRow dirRow = Program.FindDirByPath(dir);
                if (dirRow == null)
                {
                    dirRow = Program.metaDataSet.Directories.NewDirectoriesRow();
                    dirRow.Path = dir;
                    dirRow.Name = Misc.GetDirectoryNameFromPath(dir);
                    Program.metaDataSet.Directories.AddDirectoriesRow(dirRow);
                }

                DirectoryInfo di = new DirectoryInfo(dir);
                foreach (FileInfo fi in di.GetFiles())
                {
                    MetaDataSet.FilesRow dfRow = Program.FindFileByPath(dirRow.ID, fi.FullName);
                    if (dfRow == null)
                    {
                        dirRow.FileCount = dirRow.FileCount + 1;
                        dfRow = Program.metaDataSet.Files.NewFilesRow();
                        dfRow.DirectoryID = dirRow.ID;
                        dfRow.Path = fi.FullName;
                        dfRow.Name = Misc.GetFileNameFromPath(fi.FullName, false);
                        Program.metaDataSet.Files.AddFilesRow(dfRow);
                    }
                }
                Program.DataSave("files_" + dirRow.ID);
                foreach (DirectoryInfo ddi in di.GetDirectories())
                {
                    Add("dir", ddi.FullName, true);
                }
                if (!childDirectory)
                {
                    Program.DataSave("dirs");                    
                }
            }
        }

        public void CloseTabPage(TabPage tabPage)
        {
            if (tabPage != null)
            {
                if (tabPage is TabPageEx)
                {
                    TabPageEx tpEx = (TabPageEx)tabPage;
                    if (tpEx.Control != null)
                    {
                        tpEx.Control.Dispose();
                    }
                }
                mainTabControl.TabPages.Remove(tabPage);
                tabPage.Dispose();
            }
        }

        public void UpdateTabImages()
        {
            foreach (TabPage tabPage in mainTabControl.TabPages)
            {
                if (tabPage is TabPageEx)
                {
                    TabPageEx tabPageEx = (TabPageEx) tabPage;
                    tabPageEx.TabPageImageKey = tabPageEx.TabPageImageKey;
                }
            }
        }

        private TabPageEx FindTabPage(Type controlType)
        {
            foreach (TabPage tabPage in mainTabControl.TabPages)
            {
                if (tabPage is TabPageEx)
                {
                    TabPageEx tabPageEx = (TabPageEx) tabPage;
                    if (tabPageEx.Control.GetType() == controlType)
                    {
                        return tabPageEx;
                    }
                    else if (tabPageEx.Control.GetType().IsSubclassOf(controlType))
                    {
                        return tabPageEx;
                    }
                }
            }
            return null;
        }        

        public TabPageEx OpenUserControl(Type controlType, bool openNew = false)
        {
            TabPageEx tabPage = null;
            if (!openNew)
                tabPage = FindTabPage(controlType);

            if (tabPage == null)
            {
                if (controlType.IsSubclassOf(typeof (UserControlBase)))
                {
                    ConstructorInfo constInfo = controlType.GetConstructor(new Type[0]);
                    UserControlBase cont = (UserControlBase)constInfo.Invoke(new object[0]);

                    tabPage = new TabPageEx(cont);

                    mainTabControl.TabPages.Add(tabPage);                    
                }
            }
            
            if (tabPage != null)
                mainTabControl.SelectedTab = tabPage;

            closeTabPageLabel.Visible = tabPage != null;

            return tabPage;
        }

        #endregion

        #region Form Events
        private void explorerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenUserControl(typeof(ExplorerUserControl));
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            explorerToolStripMenuItem.PerformClick();
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenUserControl(typeof (OptionsUserControl));
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
            {
                if (e.Shift)
                {
                    confirmExit = false;
                }
                Close();                
            }
        }

        private void mainTabControl_MouseUp(object sender, MouseEventArgs e)
        {
            var tabControl = sender as TabControl;
            TabPage tabPageCurrent = null;
            if (e.Button == MouseButtons.Middle)
            {
                for (var i = 0; i < tabControl.TabCount; i++)
                {
                    if (!tabControl.GetTabRect(i).Contains(e.Location))
                        continue;
                    tabPageCurrent = tabControl.TabPages[i];
                    break;
                }
                
                CloseTabPage(tabPageCurrent);
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!confirmExit)
                return;

            if (MessageBox.Show("Are you sure you want to exit application?", "MetaFiler", MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.No)
            {
                e.Cancel = true;
            }
        }

        private void tagsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenUserControl(typeof(TagsUserControl));
        }

        private void mainTabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            closeTabPageLabel.Visible = mainTabControl.SelectedTab != null;
        }

        private void closeTabPageLabel_Click(object sender, EventArgs e)
        {
            CloseTabPage(mainTabControl.SelectedTab);
        }

        #endregion

        #region Override

        protected override unsafe void WndProc(ref Message m)
        {
            // Primanje poruke od drugog processa...

            if (m.Msg == (int) Win32.WindowsMessage.COPYDATA)
            {
                Win32.COPYDATASTRUCT data = (Win32.COPYDATASTRUCT) m.GetLParam(typeof (Win32.COPYDATASTRUCT));
                string str = new string((char*) (data.lpData), 0, data.cbData/2);
                string[] strp = str.Split(';');
                if (strp.Length > 0)
                {
                    if (strp[0] == "add")
                    {
                        string addType = strp[1];
                        string addPath = strp[2];

                        Add(addType, addPath);
                    }
                }
                m.Result = new IntPtr(Win32.TRUE);
            }
            else
            {
                base.WndProc(ref m);   
            }
        }

        #endregion
    }
}
