﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using MetaFiler.Data;
using MetaFiler.Forms;
using MetaFiler.Utils;

namespace MetaFiler
{
    static class Program
    {
        #region Static Variables

        public static string exePath;
        public static string dataPath;
        public static MainForm mainForm;

        public static MetaDataSet metaDataSet;

        #endregion
        
        #region Methods

        public static MetaDataSet.FileTagsRow FindFileTagsRow(int fileID, int tagID)
        {
            DataRow[] fRows = metaDataSet.FileTags.Select("FileID = " + fileID + " AND TagID = " + tagID);
            if (fRows.Length > 0)
                return (MetaDataSet.FileTagsRow)fRows[0];

            return null;
        }


        public static MetaDataSet.DirectoryTagsRow FindDirectoryTagsRow(int dirID, int tagID)
        {
            DataRow[] fRows = metaDataSet.DirectoryTags.Select("DirectoryID = " + dirID + " AND TagID = " + tagID);
            if (fRows.Length > 0)
                return (MetaDataSet.DirectoryTagsRow) fRows[0];
            
            return null;
        }

        public static MetaDataSet.FilesRow FindFileByPath(string path)
        {
            DataRow[] ffRows = Program.metaDataSet.Files.Select("Path = '" + path + "'");
            if (ffRows.Length > 0)
            {
                return (MetaDataSet.FilesRow)ffRows[0];
            }

            return null;
        }

        public static MetaDataSet.FilesRow FindFileByPath(int dirID, string path)
        {
            DataRow[] ffRows = Program.metaDataSet.Files.Select("DirectoryID = " + dirID + " AND Path = '" + path.Replace("'", "''") + "'");
            if (ffRows.Length > 0)
            {
                return (MetaDataSet.FilesRow)ffRows[0];
            }

            return null;
        }

        public static MetaDataSet.DirectoriesRow FindDirByPath(string path)
        {
            DataRow[] fdRows = Program.metaDataSet.Directories.Select("Path = '" + path.TrimEnd('\\') + "'");
            if (fdRows.Length > 0)
            {
                return (MetaDataSet.DirectoriesRow) fdRows[0];
            }

            return null;
        }

        public static void DataLoad(string dataType)
        {
            Debug.WriteLine("DataLoad: " + dataType);

            switch (dataType.ToLower())
            {
                case "tags":
                    string tagsFilePath = dataPath + "tags.xml";
                    if (File.Exists(tagsFilePath))
                    {
                        Misc.ReadDataTableToXml(metaDataSet.Tags, tagsFilePath);
                        //metaDataSet.Tags.ReadXml(tagsFilePath);
                    }
                    break;
                case "dirs":
                case "directories":
                    string dirsFilePath = dataPath + "dirs.xml";
                    if (File.Exists(dirsFilePath))
                    {
                        //metaDataSet.Directories.ReadXml(dirsFilePath);
                        Misc.ReadDataTableToXml(metaDataSet.Directories, dirsFilePath);
                    }
                    string dirsTagsFilePath = dataPath + "dirs_tags.xml";
                    if (File.Exists(dirsTagsFilePath))
                    {
                        //metaDataSet.DirectoryTags.ReadXml(dirsTagsFilePath);
                        Misc.ReadDataTableToXml(metaDataSet.DirectoryTags, dirsTagsFilePath);
                    }
                    break;
            }

            if (dataType.StartsWith("files"))
            {
                bool isFilesTags = dataType.StartsWith("filestags");

                if (dataType != "files" && !isFilesTags)
                {
                    int dirID = Convert.ToInt32(dataType.Substring(6));
                    string dfPath = dataPath + "dir_" + dirID + "_files.xml";
                    DataRow[] cdfRows = metaDataSet.Files.Select("DirectoryID = " + dirID);
                    foreach (DataRow cdfRow in cdfRows)
                    {
                        cdfRow.Delete();
                    }
                    metaDataSet.Files.AcceptChanges();
                    int fcc = metaDataSet.Files.Columns.Count;
                    if (File.Exists(dfPath))
                    {
                        using (MetaDataSet.FilesDataTable filesTable = new MetaDataSet.FilesDataTable())
                        {
                            //filesTable.ReadXml(dfPath);
                            Misc.ReadDataTableToXml(filesTable, dfPath);

                            foreach (MetaDataSet.FilesRow rfRow in filesTable)
                            {
                                metaDataSet.Files.ImportRow(rfRow);
                            }
                        }                        
                    }
                }
                else if (isFilesTags)
                {
                    int dirID = Convert.ToInt32(dataType.Substring(10));
                    string dfPath = dataPath + "dir_" + dirID + "_ftags.xml";
                    if (File.Exists(dfPath))
                    {
                        using (MetaDataSet.FileTagsDataTable fTagsTable = new MetaDataSet.FileTagsDataTable())
                        {
                            //fTagsTable.ReadXml(dfPath);
                            Misc.ReadDataTableToXml(fTagsTable, dfPath);

                            foreach (MetaDataSet.FileTagsRow ftRow in fTagsTable)
                            {
                                DataRow[] cftRows = metaDataSet.FileTags.Select("FileID = " + ftRow.FileID + " AND TagID = " + ftRow.TagID);
                                
                                foreach (DataRow cftRow in cftRows)
                                    cftRow.Delete();

                                metaDataSet.FileTags.ImportRow(ftRow);
                            }
                        }                        
                    }
                }
            }
        }

        public static void DataSave(string dataType)
        {
            Debug.WriteLine("DataSave: " + dataType);

            switch (dataType.ToLower())
            {
                case "tags":
                    string tagsFilePath = dataPath + "tags.xml";
                    Misc.WriteDataTableToXml(metaDataSet.Tags, tagsFilePath);
                    //metaDataSet.Tags.WriteXml(tagsFilePath, XmlWriteMode.IgnoreSchema);
                    break;
                case "dirs":
                case "directories":
                    string dirsFilePath = dataPath + "dirs.xml";
                    //metaDataSet.Directories.WriteXml(dirsFilePath, XmlWriteMode.IgnoreSchema);
                    Misc.WriteDataTableToXml(metaDataSet.Directories, dirsFilePath);
                    string dirsTagsFilePath = dataPath + "dirs_tags.xml";
                    //metaDataSet.DirectoryTags.WriteXml(dirsTagsFilePath, XmlWriteMode.IgnoreSchema);
                    Misc.WriteDataTableToXml(metaDataSet.DirectoryTags, dirsTagsFilePath);
                    break;
            }

            if (dataType.StartsWith("files"))
            {
                MetaDataSet.DirectoriesRow[] dirRows;

                bool isFilesTags = dataType.StartsWith("filestags");

                if (dataType == "files" || dataType == "filestags")
                {
                    dirRows = new MetaDataSet.DirectoriesRow[metaDataSet.Directories.Count];
                    metaDataSet.Directories.Rows.CopyTo(dirRows, 0);
                }
                else
                {
                    int dirID;
                    if (isFilesTags)
                    {
                        dirID = Convert.ToInt32(dataType.Substring(10));
                    }
                    else
                    {
                        dirID = Convert.ToInt32(dataType.Substring(6));
                    }
                    dirRows = new MetaDataSet.DirectoriesRow[1];
                    dirRows[0] = metaDataSet.Directories.FindByID(dirID);
                    if (dirRows[0] == null)
                    {
                        string dirFilesFilePath = dataPath + "dir_" + dirID + "_files.xml";
                        if (File.Exists(dirFilesFilePath))
                        {
                            File.Delete(dirFilesFilePath);
                        }
                        string dirFilesTagsFilePath = dataPath + "dir_" + dirID + "_ftags.xml";
                        if (File.Exists(dirFilesTagsFilePath))
                        {
                            File.Delete(dirFilesTagsFilePath);
                        }
                        return;
                    }
                }
                    
                foreach (MetaDataSet.DirectoriesRow dirRow in dirRows)
                {
                    if (!isFilesTags)
                    {
                        string filesFilePath = dataPath + "dir_" + dirRow.ID + "_files.xml";
                        using (MetaDataSet.FilesDataTable filesTable = new MetaDataSet.FilesDataTable())
                        {
                            int cic = filesTable.Columns.Count;
                            foreach (MetaDataSet.FilesRow fRow in dirRow.GetFilesRows())
                            {
                                MetaDataSet.FilesRow nfRow = filesTable.NewFilesRow();
                                nfRow.ID = fRow.ID;
                                filesTable.AddFilesRow(nfRow);

                                for (int ii = 1; ii < cic; ii++)
                                    nfRow[filesTable.Columns[ii]] = fRow[metaDataSet.Files.Columns[ii]];

                            }
                            //filesTable.WriteXml(filesFilePath, XmlWriteMode.IgnoreSchema);
                            Misc.WriteDataTableToXml(filesTable, filesFilePath);
                        }
                    }
                    else
                    {
                        string filesTagsFilePath = dataPath + "dir_" + dirRow.ID + "_ftags.xml";
                        using (MetaDataSet.FileTagsDataTable fTagsTable = new MetaDataSet.FileTagsDataTable())
                        {
                            int cic = fTagsTable.Columns.Count;
                            foreach (MetaDataSet.FilesRow fRow in dirRow.GetFilesRows())
                            {
                                foreach (MetaDataSet.FileTagsRow ftRow in fRow.GetFileTagsRows())
                                {
                                    fTagsTable.ImportRow(ftRow);
                                }
                            }
                            //fTagsTable.WriteXml(filesTagsFilePath, XmlWriteMode.IgnoreSchema);
                            Misc.WriteDataTableToXml(fTagsTable, filesTagsFilePath);
                        }                        
                    }
                }                    
                


            }
        }

        #endregion

        #region Main

        [STAThread]
        private static void Main(string[] args)
        {
            // Napravi pathove od aplikacije...

            exePath = Application.ExecutablePath;
            exePath = exePath.Substring(0, exePath.LastIndexOf('\\')) + '\\';

            dataPath = exePath + "data";
            if (!Directory.Exists(dataPath))
                Directory.CreateDirectory(dataPath);

            dataPath += "\\";

            // Pronadji vec postojeci process...

            Process curProc = Process.GetCurrentProcess();
            Process mfProcess = null;
            foreach (Process cpr in Process.GetProcesses())
            {
                if (cpr.ProcessName.Contains("MetaFiler"))
                {
                    if (cpr.Id != curProc.Id)
                    {
                        if (cpr.MainWindowTitle == "MetaFiler")
                        {
                            mfProcess = cpr;
                            break;
                        }
                    }
                }
            }

            // Parsiraj argumente komandne linije...

            string addType = "";
            string addPath = "";

            string argStr = "";
            for (var ai = 0; ai < args.Length; ai++)
            {
                argStr += args[ai] + "\n";
            }

            if (args.Length > 0)
            {
                foreach (string cmda in args)
                {
                    if (cmda.StartsWith("-"))
                    {
                        int eqInd = cmda.IndexOf('=');
                        if (eqInd > 0)
                        {
                            string argName = cmda.Substring(1, eqInd - 1);
                            string argValue = cmda.Substring(eqInd + 1);

                            switch (argName)
                            {
                                case "at":
                                    addType = argValue;
                                    break;
                                case "add":
                                    addPath = argValue;
                                    break;
                            }
                        }
                    }
                }
            }

            if (mfProcess != null)
            {
                // Ako je vec pokrenut process, posalji komandu preko windows api-a i zatvori trenuti process...
                // Pogledaj na MainForm-u override WndProc metode...

                if (addType.Length > 0 && addPath.Length > 0)
                {
                    Process mfProc = mfProcess;

                    string addStr = "add;" + addType + ";" + addPath;
                    IntPtr addStrPtr = Marshal.StringToHGlobalUni(addStr);

                    Win32.COPYDATASTRUCT cd = new Win32.COPYDATASTRUCT()
                    {
                        dwData = 0,
                        cbData = addStr.Length*2,
                        lpData = addStrPtr
                    };

                    IntPtr lpAddStruct = Marshal.AllocHGlobal(Marshal.SizeOf(cd));

                    Marshal.StructureToPtr(cd, lpAddStruct, false);
                    Win32.SendMessage(mfProc.MainWindowHandle, (int) Win32.WindowsMessage.COPYDATA, Process.GetCurrentProcess().MainWindowHandle, lpAddStruct);
                    Marshal.FreeHGlobal(lpAddStruct);
                }
            }
            else
            {
                // Ako nema processa pokrenutog, onda napravi glavnu formu, ucitaj podatke i otvori applikaciju...

                metaDataSet = new MetaDataSet();

                metaDataSet.DataTypes.AddDataTypesRow("String");
                metaDataSet.DataTypes.AddDataTypesRow("Integer");
                metaDataSet.DataTypes.AddDataTypesRow("Float");

                DataLoad("tags");
                DataLoad("dirs");

                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);

                mainForm = new MainForm();

                if (addType.Length > 0 && addPath.Length > 0)
                {
                    mainForm.Add(addType, addPath);
                }

                Application.Run(mainForm);
            }
        }

        #endregion
    }
}
