﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;

namespace MetaFiler.Utils
{
    public static class Misc
    {
        public static void WriteDataTableToXml(DataTable table, string xmlFile)
        {
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Encoding = Encoding.UTF8;
            settings.Indent = true;
            settings.NewLineHandling = NewLineHandling.Entitize;

            using (XmlWriter xmlWriter = XmlWriter.Create(xmlFile, settings))
            {
                table.WriteXml(xmlWriter);                
                xmlWriter.Close();
            }
        }
        public static void ReadDataTableToXml(DataTable table, string xmlFile)
        {
            if (File.Exists(xmlFile))
            {
                using (XmlReader xmReader = XmlReader.Create(xmlFile, new XmlReaderSettings()
                {

                }))
                {
                    table.ReadXml(xmReader);
                    xmReader.Close();
                }                
            }
        }

        public static bool ConfirmItemDelete(string delName = "item", bool ignore = false)
        {
            if (ignore)
                return true;

            return
                MessageBox.Show("Are you sure you want to delete " + delName + "?", "MetaFiler Explorer",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) ==
                DialogResult.Yes;
        }


        public static int LimitValue(int val, int min, int max)
        {
            if (val < min)
                return min;
            if (val > max)
                return max;
            return val;
        }

        public static float LimitValue(float val, float min, float max)
        {
            if (val < min)
                return min;
            if (val > max)
                return max;
            return val;
        }


        public static string GetDirectoryFromPath(string path)
        {
            string dir = path;            
            int lpi = dir.LastIndexOf('\\');
            if (lpi > 0)
                dir = dir.Substring(0, lpi);
            return dir;
        }

        public static string GetDirectoryNameFromPath(string path)
        {
            string dir = GetDirectoryFromPath(path.TrimEnd('\\') + '\\');

            int lpi = dir.LastIndexOf('\\');
            if (lpi > 0)
                dir = dir.Substring(lpi + 1);

            return dir;
        }

        public static string GetFileNameFromPath(string path, bool stripExtension = true)
        {
            string fn = path;

            int lpi = path.LastIndexOf('\\');
            if (lpi >= 0)
                fn = path.Substring(lpi + 1);

            if (stripExtension)
            {
                int lei = fn.LastIndexOf('.');
                if (lei >= 0)
                    fn = fn.Substring(0, lei);
            }

            return fn;
        }

        public static string GetFileExtensionFromPath(string path)
        {
            string fe = "";
            int lei = path.LastIndexOf('.');
            if (lei >= 0)
                fe = path.Substring(lei);
            return fe;
        }
    }
}